<?php
/**
 * Drwho controller for show characters
 * 
 * Accepts an array of configuration values to the constructor. If no config 
 * passed, it will automatically determine the component and controller names.
 * Internally, sets the $database, $user, $view, and component $config.
 * 
 * Executable tasks are determined by method name. All public methods that end in 
 * "Task" (e.g., displayTask, editTask) are callable by the end user.
 * 
 * View name defaults to controller name with layout defaulting to task name. So,
 * a $controller of "One" and a $task of "two" will map to:
 *
 *    /{component name}
 *        /views
 *            /one
 *                /tmpl
 *                    /two.php
 */
class DrwhoControllerShow extends \Hubzero\Component\SiteController
{
	/**
	 * Default task. Displays a list of characters.
	 *
	 * @return	void
	 */
	public function displayTask()
	{
		// Get our table class
		// This is the entry point to the database and the 
		// table of characters we'll be retrieving data from

		// NOTE:
		// The JFactory::getDBO() object is auto-assigned to $this->database 
		// when calling execute() on the controller.
		$tbl = new DrwhoTableCharacter($this->database);

		// NOTE:
		// A \Hubzero\Component\View object is auto-created when calling
		// execute() on the controller. By default, the view directory is 
		// set to the controller name and layout is set to task name.
		//
		// controller=foo&task=bar   loads a view from:
		//
		//   view/
		//     foo/
		//       tmpl/
		//         bar.php
		//
		// A new layout or name can be chosen by calling setLayout('newlayout')
		// or setName('newname') respectively.

		// Incoming filters
		$this->view->filters = array(
			'limit' => JRequest::getInt('limit', 25),
			'start' => JRequest::getInt('limitstart', 0)
		);

		// Get a total record count
		// This is used for pagination and detemrining the total number of pages
		$this->view->total = $tbl->find('count', $this->view->filters);

		// Get a list of records
		$this->view->records = $tbl->find('results', $this->view->filters);

		// Output the view
		$this->view->display();
	}

	/**
	 * Display a form for editing or creating a character entry.
	 *
	 * @param   object $tbl DrwhoTableCharacter
	 * @return	void
	 */
	public function editTask($tbl=null)
	{
		// NOTE:
		// The JFactory::getUser() object is auto-assigned to $this->juser 
		// when calling execute() on the controller.

		// Only logged in users!
		if ($this->juser->get('guest'))
		{
			return $this->displayTask();
		}

		// Make sure we load the edit view.
		// Thsi is for cases where saveTask() might encoutner a data
		// validation error and fall through to editTask(). Since layout 
		// is auto-assigned the task name, the layout will be 'save' but
		// saveTask has no layout!
		$this->view->setLayout('edit');

		// If we're being passed an object, use it instead
		// Thsi means we came from saveTask() and some error occurred.
		// Most likely a missing or incorrect field.
		if ($tbl instanceof JTable)
		{
			$this->view->entry = $tbl;
		}
		else
		{
			// Grab the incoming ID and load the record for editing
			$id = JRequest::getInt('id', 0);

			$this->view->entry = new DrwhoTableCharacter($this->database);
			$this->view->entry->load($id);
		}

		// Pass any received errors to the view
		// These will be coming from the editTask()
		if ($this->getError())
		{
			foreach ($this->getErrors() as $error)
			{
				$this->view->setError($error);
			}
		}

		// Output the view
		$this->view->display();
	}

	/**
	 * Save a character entry to the database and redirect back to
	 * the main view
	 *
	 * @return	void
	 */
	public function saveTask()
	{
		// [SECURITY] This is a Cross-Site Request Forgery token check
		JRequest::checkToken() or jexit('Invalid token');

		// Incoming data, specifically from POST
		$data = JRequest::getVar('entry', array(), 'post');

		$tbl = new DrwhoTableCharacter($this->database);

		// Bind the incoming data to the table object
		if (!$tbl->bind($data))
		{
			$this->setError($tbl->getError());
			$this->editTask($tbl);
			return;
		}

		// Pass the data to any plugins that be respond to the 
		// onBeforeSave event. This is where extra data manipulation 
		// or validation can be done.
		JPluginHelper::importPlugin('drwho');
		$dispatcher = JDispatcher::getInstance();
		$dispatcher->trigger('onBeforeSave', array($tbl));

		// Validate data
		if (!$tbl->check())
		{
			$this->setError($tbl->getError());
			$this->editTask($tbl);
			return;
		}

		// Save the data
		if (!$tbl->store())
		{
			$this->setError($tbl->getError());
			$this->editTask($tbl);
			return;
		}

		// Redirect back to the main listing
		$this->setRedirect(
			JRoute::_('index.php?option=' . $this->_option),
			JText::_('Record saved!'),
			'passed'
		);
	}
}